package com.link.core.basic.service.snowflake;

import com.link.core.cllog.LCLogger;
import com.link.core.util.CoreYmlConstants;
import com.link.core.util.DateUtil;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;

public class KeyGenerateUtil {
   private static final Logger logger = LogManager.getLogger(KeyGenerateUtil.class);
   private static volatile KeyGenerateUtil instance;
   private JdbcTemplate jdbcTemplate;
   private String queryIdSql = "select currentvalue from lnk_seq where seqname = 'S_SEQUENCE_S'";
   private String updateIdSqlPrefix = "update lnk_seq set currentvalue = ";
   private String updateIdSqlSuffix = " where seqname = 'S_SEQUENCE_S' and currentvalue = ";
   private List<String> sequenceIdList;
   private List<String> sequenceIdListBuffer;
   private List<String> orderCodeList;
   private List<String> orderCodeListBuffer;
   private int step = 2000;

   public static Long keyGenerate() {
      IdWorkerSingleton idWorker = IdWorkerSingleton.getInstance(CoreYmlConstants.getDataCenterId(), CoreYmlConstants.getMachineId());
      Long id = idWorker.nextId();
      return id;
   }

   private KeyGenerateUtil(JdbcTemplate jdbcTemplate) {
      this.jdbcTemplate = jdbcTemplate;
      this.sequenceIdList = new ArrayList();
      this.sequenceIdListBuffer = new ArrayList();
   }

   public static KeyGenerateUtil getInstance(JdbcTemplate jdbcTemplate) {
      if (instance == null) {
         Class var1 = KeyGenerateUtil.class;
         Class var2 = KeyGenerateUtil.class;
         synchronized(KeyGenerateUtil.class) {
            if (instance == null) {
               instance = new KeyGenerateUtil(jdbcTemplate);
            }
         }
      }

      return instance;
   }

   public List<String> getOrderCodeListBySize(int size) {
      boolean flag = false;
      ArrayList idList = new ArrayList();

      while(true) {
         Long currentId;
         int result;
         do {
            if (flag) {
               return idList;
            }

            currentId = (Long)this.jdbcTemplate.queryForObject(this.queryIdSql, Long.class);
            Long afterId = currentId + (long)size;
            StringBuilder sql = new StringBuilder();
            sql.append(this.updateIdSqlPrefix);
            sql.append(afterId);
            sql.append(this.updateIdSqlSuffix);
            sql.append(currentId);
            result = this.jdbcTemplate.update(sql.toString());
         } while(result <= 0);

         flag = true;

         for(int i = 1; i <= size; ++i) {
            idList.add(getGenerateOrderCode(currentId + (long)i));
         }
      }
   }

   public static String getGenerateOrderCode(Long currentId) {
      String res = "";
      String year = getSysDate();
      res = year + currentId;
      return res;
   }

   public static String getSysDate() {
      Calendar date = Calendar.getInstance();
      String year = String.valueOf(date.get(1));
      String month = String.valueOf(date.get(2));
      String day = String.valueOf(date.get(5));
      return year + month + day;
   }

   public synchronized String getOrderCode() {
      String id = "noMatchId";
      if (this.orderCodeList != null && this.orderCodeList.size() > 0) {
         id = (String)this.orderCodeList.get(0);
         this.orderCodeList.remove(0);
      } else if (this.orderCodeListBuffer != null && this.orderCodeListBuffer.size() > 0) {
         id = (String)this.orderCodeListBuffer.get(0);
         this.orderCodeListBuffer.remove(0);
         this.orderCodeList = this.orderCodeListBuffer;
         this.orderCodeListBuffer = new ArrayList();
      } else {
         this.orderCodeList = this.getOrderCodeListBySize(this.step);
         id = (String)this.orderCodeList.get(0);
         this.orderCodeList.remove(0);
      }

      if (this.orderCodeListBuffer == null || this.orderCodeListBuffer.size() <= 0) {
         LCLogger.info().withMessageKey("getOrderCode").withMessage("开始生成备用订单编号时间：" + DateUtil.dateToStr(new Date(), "HHmmss_SSS")).flush();
         this.orderCodeListBuffer = this.getOrderCodeListBySize(this.step);
         LCLogger.info().withMessageKey("getOrderCode").withMessage("生成备用订单编号结束时间：" + DateUtil.dateToStr(new Date(), "HHmmss_SSS")).flush();
      }

      return id;
   }
}
